;________________________________________________________________________
;
;                       W M R 9 1 8 C T . A S M
;________________________________________________________________________
;
; WMR-918 Record WX code - by Max IK3SVW - February 2006
; Based on original u2k package by Byon Garrabrant, N6BG
;________________________________________________________________________
	cblock
		hex1
		hex2
		hex3
		timeM
		timeH
		timeD
		directionH
		directionL
		speedL_barovar
		speedH
		gustL_aux
		gustH
		tempL
		tempH
		baroHex
		baro_seaL
		baro_seaH
		humidL
		raintotL
		raintotH
		offsetRtotL
		offsetRtotH
		rainPL
		rainPH
		rain1hL
		rain1hH
		rainrL
		rainrH
		bhist1
		bhist2
		bhist3
		bhist4
		barotrend
		auxbaro	
		auxrain

		snowflag

		checksum
	endc
;________________________________________________________________________

mul16	skpz			; 16 bit multiplication by w
	 goto mul161		; skip if multiply by zero.
	return
mul161	movwf	hex3		; no zero-check entry point
sum16	movfw	temp		; 16-bit sum of hex1&hex2 + temp&temp2
	addwf	hex1, f		; result in hex1-hex2, temp-temp2 ummodified
	skpnc			; skip if result of hex1+temp is <256
	 incf	hex2, f
	movfw	temp2	
	addwf	hex2, f
	decfsz	hex3, f		; check if continue multiplication
	 goto sum16
	return
;________________________________________________________________________

dec1hex	movwf	hex1		; convert 1 byte (2 dec nibbles) in hex
	andlw	H'F0'		; take high nibble
	movwf	hex2
	clrc			; clear carry
	rrf	hex2,w
	movwf 	hex3
	rrf	hex2,f
	rrf	hex2,f
	rrf	hex2,w
	addwf	hex3,f		; convert to decimal
	movf	hex1,w
	andlw	H'0F'		; take low nibble
	addwf	hex3,w		; combine low nibble with converted high nibble
	return
;________________________________________________________________________

neg_temp			; make 16-bit temperature negative
	comf	tempL, f
	comf	tempH, f
	incf	tempL, f
	skpnz
	 incf	tempH, f	; make temperature negative
	return
;________________________________________________________________________

serLoop	movwf	hex1		; read w bytes from serial port w/checksum
serL1	call 	serialReceive
	addwf	checksum,f	; perform checksum count
	decfsz	hex1, f
	 goto	serL1
	return
;________________________________________________________________________

serialReceiveCk2		; read two bytes from serial port w/checksum
	call	serialReceiveCk
serialReceiveCk			; read one byte from serial port w/checksum
	call	serialReceive
	addwf	checksum,f	; perform checksum count
	return
;________________________________________________________________________

Decode00			; extract paramters from serial stream
				; group byte (0x00) has already been heard
	call	serialReceiveCk2
	movwf	directionL
	swapf	directionL, f
	call	serialReceiveCk	; byte 3
	movwf	hex1
	andlw	0x0F
	movwf	directionH	; wind direction
	swapf	hex1, w
	andlw	0x0F
	movwf	gustL_aux
	call	serialReceiveCk	; byte 4
	movwf	hex1
	swapf	hex1, w
	andlw	0xF0
	addwf	gustL_aux, f	; wind gust
	swapf	hex1, w
	andlw	0x0F
	movwf	gustH
	
	call	serialReceiveCk	; byte 5
	movwf	speedL_barovar	; wind speed
	call	serialReceiveCk	; byte 6
	andlw	0x0F
	movwf	speedH		; wind speed

	call	serialReceiveCk ; byte 7 (not relevant)
	call	serialReceive	; read checksum byte
	xorwf	checksum, w
	skpnz
	 bsf	auxdecoded,0	; checksum is ok

	goto	allChk
;________________________________________________________________________

Decode01			; extract paramters from serial stream
				; group byte (0x01) has already been heard

	movlw	D'4'		; skip 3 bytes and read 1 byte
	call	serLoop
	movwf	raintotL	; total rain
	call	serialReceiveCk
	movwf	raintotH

	movlw	D'7'		; skip remaining 7 bytes
	call	serLoop
	call	serialReceive	; read checksum byte
	xorwf	checksum, w
	skpnz
	 bsf	auxdecoded,1	; checksum is ok

	goto	allChk
;________________________________________________________________________

Decode03			; extract paramters from serial stream
				; group byte (0x03) has already been heard
	call	serialReceiveCk2
	movwf	tempL
	call	serialReceiveCk	; byte 3
	movwf	tempH		; temperature
	bcf	alldecoded,7
	btfsc	tempH,7		; check if negative
	 bsf	alldecoded,7
	andlw	H'07'
	movwf	tempH
	call	serialReceiveCk	; byte 4
	movwf	humidL		; humidity

	call	serialReceiveCk	; byte 5 (not relevant)
	call	serialReceive	; read checksum byte
	xorwf	checksum, w
	skpnz	
	 bsf	auxdecoded,2	; checksum is ok

	goto	allChk
;________________________________________________________________________

Decode56			; groups 5&6 common decode routine
	movlw	D'6'		; skip 5 bytes and read 1 byte
	call	serLoop
	movwf	baroHex		; baro pressure
	call	serialReceiveCk
	movwf	barotrend	; barometer trend
	call	serialReceiveCk
	btfss	auxdecoded, 6
	 call	serialReceiveCk
	movwf	baro_seaL	; sea level reference
	call	serialReceiveCk
	movwf	baro_seaH
	return
;________________________________________________________________________

Decode05			; extract paramters from serial stream
				; group byte (0x05) has already been heard
	bsf	auxdecoded,6	; Group 05 flag set
	call	Decode56
	movlw	D'4'
	movwf	hex1
Dec05sh	clrc
	rrf	baro_seaH, f	; shift one nibble sea level reference
	rrf	baro_seaL, f
	decfsz	hex1, f
	 goto	Dec05sh
	
	goto	Dec06ok
;________________________________________________________________________

Decode06			; extract paramters from serial stream
				; group byte (0x06) has already been heard
	call	Decode56
	swapf	barotrend, f
Dec06ok
	call	serialReceive	; read checksum byte
	xorwf	checksum, w
	skpnz	
	 bsf	auxdecoded,3	; checksum is ok

	goto	allChk
;________________________________________________________________________

Decode0E			; extract paramters from serial stream
				; group byte (0x0E) has already been heard
	call	serialReceiveCk
	andlw	H'7F'
	movwf	timeM		; minute
	call	serialReceive	; read checksum byte (not used)
	
	goto	allChk1		; exit data receive loop
;________________________________________________________________________

Decode0F			; extract paramters from serial stream
				; group byte (0x0F) has already been heard
	clrf	timeM		; reset minute
	call	serialReceiveCk2
	movwf	timeH		; hour
	call	serialReceiveCk
	movwf	timeD		; day
	movlw	D'2'		; skip remaining 2 bytes (month & year)
	call	serLoop
	call	serialReceive	; read checksum byte
	xorwf	checksum, w
	skpnz	
	 bsf	auxdecoded,4	; checksum is ok so set hour chime flag

	goto	allChk1		; and exit data receive loop
;________________________________________________________________________

resBaro	call	StoreBaro	; reset 4 hours baro variation
	call	Shift4Baro
	call	Shift4Baro
Shift4Baro			; shift pressure history trough 4 hours
	movfw	bhist3
	movwf	bhist4
	movfw	bhist2
	movwf	bhist3
	movfw	bhist1
	movwf	bhist2
StoreBaro			; store last pressure in history
	movfw	baroHex
	movwf	bhist1
	return
;________________________________________________________________________

CommonDecHex
	call	dec1hex
	movwf	hex1
	clrf	hex2
	movlw	D'100'
	movwf	temp
	clrf	temp2
	return
;________________________________________________________________________

mstomph				; m/s to miles/hour conversion
	movfw	hex1		; i.e. multiply by 22 (2.2 * 10)
	movwf	temp
	movfw	hex2
	movwf	temp2
	movlw	D'23'		; really multiply by 23 for rounding
	call	mul161		; integer values
	movfw	hex1
	return
;________________________________________________________________________

convertWX
;
; Fix units and modify the data to be in BCD format,
; so there are no conversion delays while transmitting.
;________________________________________________________________________

convck0	btfss	auxdecoded, 0	; convert only if valid data was received
	 goto	convck1

; wind speed conversion from Dec To Hex
	movfw	speedL_barovar
	call	CommonDecHex
	movfw	speedH
	call	mul16

; m/s to miles/hour conversion
	call	mstomph
	movwf	speedL_barovar
	movfw	hex2
	movwf	speedH

	movlw	speedL_barovar	; BCD conversion of wind speed
	movwf	fsr		; only speedH will be used in wx
	call	bin2pack	; bcn, so speedL_barovar is avaiable

	clrf	speedL_barovar	; *** TEST clear barovar ****

; wind gust conversion from Dec To Hex
	movfw	gustL_aux
	call	CommonDecHex
	movfw	gustH
	call	mul16

; m/s to miles/hour conversion
	call	mstomph
	movwf	gustL_aux
	movfw	hex2
	movwf	gustH

	movlw	gustL_aux	; BCD conversion of wind gust
	movwf	fsr		; only gustH will be used in wx
	call	bin2pack	; beacon, so gustL_aux is avaiable
;________________________________________________________________________

convck1	btfss	auxdecoded, 2	; convert only if valid data was received
	 goto	convck2

; temp conversion from Dec To Hex
	movfw	tempL
	call	CommonDecHex
	movfw	tempH
	call	mul16

; if baro received and temp below 2C set snow flag
	bcf	snowflag,0	; yes, clear snow flag
	btfsc	alldecoded,7	; negative temp (C) ?
	 goto	cksnw		; yes, set snow flag
	movfw	hex2		; no, check degrees
	skpz			; temp (C) high byte = 0 ?
	 goto	C_to_F		; no, continue
	movfw	hex1		; yes, check degrees
	sublw	D'20'
	skpnc			; temp < 2 C ?
cksnw	 bsf	snowflag,0	; yes, set snow flag

; C to F conversion 1st step: multiply by 18 (9/5 * 10)
C_to_F 	movfw	hex1
	movwf	temp
	movfw	hex2
	movwf	temp2
	movlw	D'17'
	call	mul161

	movfw	hex2		; BCD conversion of temp,
	movwf	hex3		; really used for dividing by 100
	movfw	hex1
	movwf	hex2
	movlw	hex2
	movwf	fsr
	call	bin2pack

; convert High Order Byte Temp (0-99) from Dec to Hex
	movfw	hex3
	call	dec1hex
	movwf	tempL
	clrf	tempH

; check if C temperature has negative sign.
	btfsc	alldecoded, 7
	 call	neg_temp

; C to F conversion 2nd step: add 32
	movlw	D'32'
	addwf	tempL, f
	skpnc
	 incf	tempH, f

; check if converted F temperature is negative.
	bcf	alldecoded, 7
	btfss	tempH, 7
	 goto	tewmc1		; if temp is negative then
	bsf	alldecoded, 7	; set a flag and
	call	neg_temp

tewmc1	movlw	tempL		; BCD conversion of temp
	movwf	fsr
	call	bin2pack
;________________________________________________________________________

convck2	btfss	auxdecoded, 1	; convert only if valid data was received
	 goto	convck3

; total rain conversion from Dec To Hex
	movfw	raintotH
 	call	dec1hex
	movwf	raintotH
	movfw	raintotL
	call	CommonDecHex
	movfw	raintotH
	call	mul16

; mm/h to inch/h cent conversion (multiply by 4)
	clrc
	rlf	hex1, f
	rlf	hex2, f
	clrc
 	rlf	hex1, f
	rlf	hex2, f

; store converted total rain value
  	movfw	hex1
	movwf	raintotL
	movfw	hex2
	movwf	hex3
	movwf	raintotH

; rain since last hour calculation
	movfw	rain1hL
	subwf	raintotL, w
	movwf	rainrL
	skpc
	 decf	hex2, f
	movfw	rain1hH
	subwf	hex2, w
	movwf	rainrH
	
; rain since last day (midnight) calculation
	movfw	offsetRtotL
	subwf	raintotL, w
	movwf	rainPL
	skpc
	 decf	hex3, f
	movfw	offsetRtotH
	subwf	hex3, w
	movwf	rainPH

	movlw	rainrL		; BCD conversion of 1h rain
	movwf	fsr
	call	bin2pack

	movlw	rainPL		; BCD conversion of Daily rain
	movwf	fsr
	call	bin2pack

	btfss	alldecoded, 4	; on hour chime do not check group
	btfsc	alldecoded, 1	; group check if has been heard
	 goto	convck3
	call	resRtot		; if not previouly heard reset r&P
;________________________________________________________________________

convck3	btfss	auxdecoded, 3	; convert only if valid data was received
	 return

; sea level reference conversion from Dec To Hex
	movfw	baro_seaH
 	call	dec1hex
	movwf	gustL_aux
	movfw	baro_seaL
	call	CommonDecHex
	movfw	gustL_aux
	call	mul161

; add sea level reference to baro obtaining pressure.
	movfw	baroHex
	movwf	temp
	movlw	D'1'
	movwf	hex3
	btfss	auxdecoded, 6
	 movwf	temp2
	call	sum16
	movfw	hex1
	movwf	baro_seaL
	movfw	hex2
	movwf	baro_seaH

	movlw	baro_seaL
	movwf	fsr
	call	bin2pack	; BCD conversion of press variation

; 4h pressure variation calculation
	movfw	bhist4		; get 4 hours ago pressure from history
	subwf	baroHex, w	; get press variation since 4 hours
	bcf	alldecoded, 5
	skpnc			; check press variation
	 goto	pv4h1
	bsf	alldecoded, 5	; if negative set a flag
	clrf	temp
	subwf	temp, w		; and make it positive
pv4h1	movwf	speedL_barovar	; store press variation for BCD

	movlw	speedL_barovar	; BCD conversion of press variation
	movwf	fsr
	call	bin2byte

	btfss	alldecoded, 4	; on hour chime do not check group
	btfsc	alldecoded, 3	; group check if has been heard
         return
	goto	resBaro		; if not previouly heard reset v
;________________________________________________________________________
